using GLib;
using Gtk;
using GnomeVFS;

public class Document : SourceBuffer
{
	private string file_name;
	private string mime_type;
	private string modified;
	
	public MetadataManager metadata { get; set; }


	public Document () {
		highlight = true;
	}
	
	public construct (string uri) {
		file_name = uri_to_filename (uri);
		metadata = new MetadataManager (escape_string (file_name));
		mime_type = get_mime_type_for_name (file_name);
		
		if ( mime_type != null ) {
			var slm = new SourceLanguagesManager();
			var lang = slm.get_language_from_mime_type(mime_type);

			if ( lang != null ) {
				set_language (lang);
			}
		}
	}
	
	public bool open () {
		// TODO: error dialogs
		string contents;
		
		if ( ! File.get_contents (file_name, out contents, null, null) )
			return false;
		
		if ( ! contents.validate () ) {
			// this is lame, but everyone should use UTF-8 anyway...
			contents = convert (contents, -1, "UTF-8", "ISO-8859-2", null, null, null);
			if ( ! contents.validate () ) {
				return false;
			}
		}
		
		begin_not_undoable_action ();
		set_text (contents, -1);
		end_not_undoable_action ();
		set_modified (false);
		return true;
	}
	
	public void save () {
		return_if_fail (this != null);
	
		if ( !get_modified () ) return;
		
		string buffer;
		buffer = Application.get_config_dir () + "/save_buffer";
		
		TextIter start, end;
		get_bounds (out start, out end);
		string contents = get_text (ref start, ref end, false);
		Error err;
		
		if ( !File.set_contents (buffer, contents, -1, out err) ) {
			stdout.printf ("Error: Could not save document: " + err.message);
			return;
		}
		
		if ( File.rename (buffer, file_name) == -1 ) {
			stdout.printf ("Error: Could not save document: " + err.message);
			return;
		}
		
		set_modified (false);
	}
	
	public string get_file_name () {
		return file_name;
	}
	
	public string get_mimetype () {
		return mime_type;
	}
	
	public string get_folder () {
		var parts = file_name.reverse ().split ("/", 2);
		return parts[1].reverse ();
	}
	
	public string get_basename () {
		return Path.display_basename (file_name);
	}
	
	public ref string get_title_string () {
		string folder = get_folder ();
		string home = Environment.get_home_dir ();
		if ( folder == home ) {
			folder = "";
		} else if ( folder.has_prefix (home) ) {
			folder = " (" + folder.offset (home.len ()+1) + ")";
		} else {
			folder = " (" + folder + ")";
		}
	
		return get_basename () + folder;
	}
	
	public static ref string uri_to_filename (string uri) {
		string f = uri;
	
		// only local URIs are supported for now
		if ( uri.has_prefix ("file:") )
			f = uri.split ("://")[1];
		else if ( ! Path.is_absolute (uri) )
			f = Path.build_filename (Environment.get_current_dir (), uri);
		
		return f;
	}
}
